import * as React from "react";
import * as PropTypes from "prop-types";
import { ScrollbarTrackProps } from "./ScrollbarTrack";
import { ScrollbarThumbProps } from "./ScrollbarThumb";
import Emittr from "./Emittr";
export declare type ScrollValues = {
    /**
     * @description Content's native clientHeight parameter
     */
    clientHeight: number;
    /**
     * @description Content's native clientWidth parameter
     */
    clientWidth: number;
    /**
     * @description Content's native scrollHeight parameter
     */
    scrollHeight: number;
    /**
     * @description Content's native scrollWidth parameter
     */
    scrollWidth: number;
    /**
     * @description Content's native scrollTop parameter
     */
    scrollTop: number;
    /**
     * @description Content's native scrollLeft parameter
     */
    scrollLeft: number;
    /**
     * @description Indicates whether vertical scroll blocked via properties
     */
    scrollYBlocked: boolean;
    /**
     * @description Indicates whether horizontal scroll blocked via properties
     */
    scrollXBlocked: boolean;
    /**
     * @description Indicates whether the content overflows vertically and scrolling not blocked
     */
    scrollYPossible: boolean;
    /**
     * @description Indicates whether the content overflows horizontally and scrolling not blocked
     */
    scrollXPossible: boolean;
    /**
     * @description Indicates whether vertical track is visible
     */
    trackYVisible: boolean;
    /**
     * @description Indicates whether horizontal track is visible
     */
    trackXVisible: boolean;
    /**
     * @description Indicates whether display direction is right-to-left
     */
    isRTL?: boolean;
};
export declare type ElementRef<T = HTMLElement> = (element: T | null) => void;
export declare type ElementProps<T = HTMLElement> = React.HTMLProps<T> & {
    elementRef?: ElementRef;
    renderer?: React.FunctionComponent<ElementProps>;
};
export declare enum SCROLLBAR_TRACK_CLICK_BEHAVIOR {
    JUMP = "jump",
    STEP = "step"
}
export declare type ScrollbarProps = ElementProps & {
    createContext?: boolean;
    rtl?: boolean;
    momentum?: boolean;
    native?: boolean;
    noScrollX?: boolean;
    noScrollY?: boolean;
    noScroll?: boolean;
    permanentTrackX?: boolean;
    permanentTrackY?: boolean;
    permanentTracks?: boolean;
    removeTracksWhenNotUsed?: boolean;
    removeTrackYWhenNotUsed?: boolean;
    removeTrackXWhenNotUsed?: boolean;
    minimalThumbSize?: number;
    maximalThumbSize?: number;
    minimalThumbXSize?: number;
    maximalThumbXSize?: number;
    minimalThumbYSize?: number;
    maximalThumbYSize?: number;
    scrollbarWidth?: number;
    fallbackScrollbarWidth?: number;
    scrollTop?: number;
    scrollLeft?: number;
    scrollDetectionThreshold?: number;
    translateContentSizesToHolder?: boolean;
    translateContentSizeYToHolder?: boolean;
    translateContentSizeXToHolder?: boolean;
    noDefaultStyles?: boolean;
    trackClickBehavior?: SCROLLBAR_TRACK_CLICK_BEHAVIOR;
    wrapperProps?: ElementProps;
    contentProps?: ElementProps;
    trackXProps?: Pick<ScrollbarTrackProps, Exclude<keyof ScrollbarTrackProps, "axis">>;
    trackYProps?: Pick<ScrollbarTrackProps, Exclude<keyof ScrollbarTrackProps, "axis">>;
    thumbXProps?: Pick<ScrollbarThumbProps, Exclude<keyof ScrollbarThumbProps, "axis">>;
    thumbYProps?: Pick<ScrollbarThumbProps, Exclude<keyof ScrollbarThumbProps, "axis">>;
    onUpdate?: (scrollValues: ScrollValues, prevScrollValues: ScrollValues) => void;
    onScroll?: (scrollValues: ScrollValues, prevScrollValues: ScrollValues) => void;
    onScrollStart?: (scrollValues: ScrollValues) => void;
    onScrollEnd?: (scrollValues: ScrollValues) => void;
};
export declare type ScrollbarState = {
    trackXVisible: boolean;
    trackYVisible: boolean;
    isRTL?: boolean;
};
export declare type ScrollbarContextValue = {
    parentScrollbar: Scrollbar | null;
};
export declare const ScrollbarContext: React.Context<ScrollbarContextValue>;
export default class Scrollbar extends React.Component<ScrollbarProps, ScrollbarState> {
    static contextType: React.Context<ScrollbarContextValue>;
    /**
     * @description UUID identifying scrollbar instance
     */
    readonly id: string;
    /**
     * @description Reference to the holder HTMLElement or null if it wasn't rendered or <i>native</i> property is true
     */
    holderElement: HTMLElement | null;
    /**
     * @description Reference to the wrapper HTMLElement or null if it wasn't rendered or <i>native</i> property is true
     */
    wrapperElement: HTMLElement | null;
    /**
     * @description Reference to the content HTMLElement that contains component's children (and has browser's scrollbars)
     */
    contentElement: HTMLElement | null;
    /**
     * @description Reference to the horizontal track HTMLElement or null if it wasn't rendered
     */
    trackXElement: HTMLElement | null;
    /**
     * @description Reference to the vertical track HTMLElement or null if it wasn't rendered
     */
    trackYElement: HTMLElement | null;
    /**
     * @description Reference to the horizontal thumb HTMLElement or null if it wasn't rendered
     */
    thumbXElement: HTMLElement | null;
    /**
     * @description Reference to the vertical thumb HTMLElement or null if it wasn't rendered
     */
    thumbYElement: HTMLElement | null;
    readonly eventEmitter: Emittr;
    static propTypes: {
        createContext: PropTypes.Requireable<boolean>;
        rtl: PropTypes.Requireable<boolean>;
        native: PropTypes.Requireable<boolean>;
        momentum: PropTypes.Requireable<boolean>;
        noDefaultStyles: PropTypes.Requireable<boolean>;
        minimalThumbSize: PropTypes.Requireable<number>;
        maximalThumbSize: PropTypes.Requireable<number>;
        minimalThumbXSize: PropTypes.Requireable<number>;
        maximalThumbXSize: PropTypes.Requireable<number>;
        minimalThumbYSize: PropTypes.Requireable<number>;
        maximalThumbYSize: PropTypes.Requireable<number>;
        noScrollX: PropTypes.Requireable<boolean>;
        noScrollY: PropTypes.Requireable<boolean>;
        noScroll: PropTypes.Requireable<boolean>;
        permanentTrackX: PropTypes.Requireable<boolean>;
        permanentTrackY: PropTypes.Requireable<boolean>;
        permanentTracks: PropTypes.Requireable<boolean>;
        translateContentSizesToHolder: PropTypes.Requireable<boolean>;
        translateContentSizeYToHolder: PropTypes.Requireable<boolean>;
        translateContentSizeXToHolder: PropTypes.Requireable<boolean>;
        removeTracksWhenNotUsed: PropTypes.Requireable<boolean>;
        removeTrackYWhenNotUsed: PropTypes.Requireable<boolean>;
        removeTrackXWhenNotUsed: PropTypes.Requireable<boolean>;
        trackClickBehavior: PropTypes.Requireable<SCROLLBAR_TRACK_CLICK_BEHAVIOR>;
        scrollbarWidth: PropTypes.Requireable<number>;
        fallbackScrollbarWidth: PropTypes.Requireable<number>;
        scrollDetectionThreshold: PropTypes.Requireable<number>;
        scrollTop: PropTypes.Requireable<number>;
        scrollLeft: PropTypes.Requireable<number>;
        className: PropTypes.Requireable<string>;
        wrapperProps: PropTypes.Requireable<object>;
        contentProps: PropTypes.Requireable<object>;
        trackXProps: PropTypes.Requireable<object>;
        trackYProps: PropTypes.Requireable<object>;
        thumbXProps: PropTypes.Requireable<object>;
        thumbYProps: PropTypes.Requireable<object>;
        onUpdate: PropTypes.Requireable<(...args: any[]) => any>;
        onScroll: PropTypes.Requireable<(...args: any[]) => any>;
        onScrollStart: PropTypes.Requireable<(...args: any[]) => any>;
        onScrollEnd: PropTypes.Requireable<(...args: any[]) => any>;
    };
    static defaultProps: {
        momentum: boolean;
        minimalThumbSize: number;
        fallbackScrollbarWidth: number;
        trackClickBehavior: SCROLLBAR_TRACK_CLICK_BEHAVIOR;
        scrollDetectionThreshold: number;
        wrapperProps: {};
        contentProps: {};
        trackXProps: {};
        trackYProps: {};
        thumbXProps: {};
        thumbYProps: {};
    };
    constructor(props: any);
    componentDidMount(): void;
    componentWillUnmount(): void;
    componentDidUpdate(prevProps: Readonly<ScrollbarProps>, prevState: Readonly<ScrollbarState>): void;
    /**
     * @description Current ScrollValues (cached)
     */
    private scrollValues;
    /**
     * @description Get current scroll-related values.<br/>
     * If <i>force</i> if truthy - will recalculate them instead of returning cached values.
     *
     * @return ScrollValues
     */
    getScrollValues: (force?: boolean) => ScrollValues;
    /**
     * @description Scroll to top border
     */
    scrollToTop: () => this;
    /**
     * @description Scroll to left border
     */
    scrollToLeft: () => this;
    /**
     * @description Scroll to bottom border
     */
    scrollToBottom: () => this;
    /**
     * @description Scroll to right border
     */
    scrollToRight: () => this;
    /**
     * @description Set the scrolls at given coordinates.<br/>
     * If coordinate is undefined - current scroll value will persist.
     */
    scrollTo: (x?: number | undefined, y?: number | undefined) => this;
    /**
     * @description Center the viewport at given coordinates.<br/>
     * If coordinate is undefined - current scroll value will persist.
     */
    centerAt: (x?: number | undefined, y?: number | undefined) => this;
    scrollTop: number;
    scrollLeft: number;
    readonly scrollHeight: number;
    readonly scrollWidth: number;
    readonly clientHeight: number;
    readonly clientWidth: number;
    update: (force?: boolean) => void | ScrollValues;
    private updaterNative;
    private updaterCustom;
    private elementRefHolder;
    private elementRefWrapper;
    private elementRefContent;
    private elementRefTrackX;
    private elementRefTrackY;
    private elementRefThumbX;
    private elementRefThumbY;
    private handleTrackXClick;
    private handleTrackYClick;
    private handleTrackYMouseWheel;
    private handleTrackXMouseWheel;
    private handleThumbXDrag;
    private handleThumbYDrag;
    private handleContentScroll;
    private _scrollDetectionTO;
    private _scrollDetection;
    private _scrollDetectionCallback;
    render(): React.ReactElement<any> | null;
    static calculateStyles(props: ScrollbarProps, state: ScrollbarState, scrollValues: any, scrollbarWidth: number): {
        [key: string]: React.CSSProperties;
    };
}
